package com.flyfish.framework.controller.reactive;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.domain.tree.RootTreeNode;
import com.flyfish.framework.domain.tree.TreeDomain;
import com.flyfish.framework.domain.tree.TreeQo;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 支持树形解析的controller
 *
 * @param <T>
 * @param <Q>
 */
public abstract class ReactiveTreeController<T extends TreeDomain<T>, Q extends TreeQo<T>> extends ReactiveBaseController<T, Q> {

    /**
     * 重写树查询，允许查询根节点
     *
     * @param id 主键
     * @return 结果
     */
    @GetMapping("{id}")
    @Override
    public Mono<Result<T>> get(@PathVariable String id) {
        if (TreeDomain.ROOT.equals(id)) {
            T root = RootTreeNode.instance();
            return Mono.just(Result.accept(root));
        }
        return reactiveService.getById(id).map(Result::accept).defaultIfEmpty(Result.notFound());
    }

    /**
     * 获取权限树
     *
     * @return 结果
     */
    @GetMapping("tree")
    public Mono<Result<List<T>>> getTree(Q qo) {
        qo.setRecursive(true);
        // 第一步，查询全部，并根据条件筛选
        return reactiveService.getList(qo)
                .collectList()
                .flatMap(filtered -> {
                    // 第二步，根据父id组成map
                    Map<String, List<T>> group = filtered.stream()
                            .collect(Collectors.groupingBy(p -> StringUtils.defaultIfBlank(p.getParentId(), "")));
                    // 第三步，筛选一级树深度
                    return Flux.fromIterable(filtered)
                            .filter(item -> null != item && TreeDomain.ROOT.equals(item.getParentId()))
                            .collectList()
                            // 第四步，根据父id的map填充根tree
                            .map(topList -> applyChildren(topList, group));
                })
                .map(Result::accept);
    }

    /**
     * 递归赋值儿子们
     *
     * @param children 儿子们
     * @param group    分组
     */
    @SuppressWarnings("all")
    private List<T> applyChildren(List<T> children, Map<String, List<T>> group) {
        if (CollectionUtils.isNotEmpty(children)) {
            children.forEach(child -> {
                List<T> treeNodes = group.getOrDefault(child.getId(), Collections.emptyList());
                child.setChildren(treeNodes);
                applyChildren(treeNodes, group);
            });
        }
        return children;
    }
}
